@@ -23,8 +23,6 @@
 
 #include <config.h>
 
-#include <time.h>
-
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <gio/gio.h>
@@ -37,7 +35,6 @@
 #include "gs-manager.h"
 #include "gs-prefs.h"
 #include "gs-window.h"
-#include "xfce-bg.h"
 
 static void gs_manager_class_init (GSManagerClass *klass);
 static void gs_manager_init       (GSManager      *manager);
@@ -47,8 +44,6 @@ struct GSManagerPrivate {
     GSList         *windows;
     GHashTable     *jobs;
 
-    XfceBG         *bg;
-
     /* Policy */
     GSPrefs         *prefs;
     guint           throttled : 1;
@@ -57,12 +52,9 @@ struct GSManagerPrivate {
     /* State */
     guint           active : 1;
     guint           lock_active : 1;
-    guint           saver_active : 1;
 
     guint           dialog_up : 1;
 
-    time_t          activate_time;
-
     guint           lock_timeout_id;
     guint           cycle_timeout_id;
 
@@ -272,22 +264,8 @@ gs_manager_set_throttled (GSManager *manager,
 }
 
 void
-gs_manager_get_lock_active (GSManager *manager,
-                            gboolean  *lock_active) {
-    if (lock_active != NULL) {
-        *lock_active = FALSE;
-    }
-
-    g_return_if_fail (GS_IS_MANAGER (manager));
-
-    if (lock_active != NULL) {
-        *lock_active = manager->priv->lock_active;
-    }
-}
-
-void
-gs_manager_set_lock_active (GSManager *manager,
-                            gboolean   lock_active) {
+gs_manager_enable_locker (GSManager *manager,
+                          gboolean   lock_active) {
     GSList *l;
 
     g_return_if_fail (GS_IS_MANAGER (manager));
@@ -303,42 +281,10 @@ gs_manager_set_lock_active (GSManager *manager,
     }
 }
 
-void
-gs_manager_get_saver_active (GSManager *manager,
-                             gboolean  *saver_active) {
-    if (saver_active != NULL) {
-        *saver_active = FALSE;
-    }
-
-    g_return_if_fail (GS_IS_MANAGER (manager));
-
-    if (saver_active != NULL) {
-        *saver_active = manager->priv->saver_active;
-    }
-}
-
-void
-gs_manager_set_saver_active (GSManager *manager,
-                             gboolean   saver_active) {
-    GSList *l;
-
-    g_return_if_fail (GS_IS_MANAGER (manager));
-
-    gs_debug ("Setting saver active: %d", saver_active);
-
-    if (manager->priv->saver_active != saver_active) {
-        manager->priv->saver_active = saver_active;
-    }
-
-    for (l = manager->priv->windows; l; l = l->next) {
-        gs_window_set_saver_active(l->data, saver_active);
-    }
-}
-
 static gboolean
 activate_lock_timeout (GSManager *manager) {
-    gs_debug ("Locking screen after idling timeout");
-    gs_manager_set_lock_active (manager, TRUE);
+    gs_debug ("Locking screen on idle timeout");
+    gs_manager_enable_locker (manager, TRUE);
     manager->priv->lock_timeout_id = 0;
 
     return FALSE;
@@ -355,18 +301,15 @@ remove_lock_timer (GSManager *manager) {
 static void
 add_lock_timer (GSManager *manager,
                 glong      timeout) {
-    gboolean locked;
-
     if (!manager->priv->prefs->lock_enabled)
         return;
     if (!manager->priv->prefs->lock_with_saver_enabled)
         return;
-    gs_manager_get_lock_active (manager, &locked);
-    if (locked)
+    if (manager->priv->lock_active)
         return;
 
-    gs_debug ("Scheduling screen lock after screensaver is idling for %i sec", timeout / 1000);
-    manager->priv->lock_timeout_id = g_timeout_add (timeout,
+    gs_debug ("Scheduling screen lock after screensaver is idling for %i sec", timeout);
+    manager->priv->lock_timeout_id = g_timeout_add (timeout * 60000,
                                                     (GSourceFunc)activate_lock_timeout,
                                                     manager);
 }
@@ -452,7 +395,7 @@ gs_manager_set_property (GObject      *object,
             gs_manager_set_status_message (self, g_value_get_string (value));
             break;
         case PROP_ACTIVE:
-            gs_manager_set_active (self, g_value_get_boolean (value));
+            gs_manager_activate_saver (self, g_value_get_boolean (value));
             break;
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -563,12 +506,8 @@ gs_manager_init (GSManager *manager) {
 
     manager->priv->grab = gs_grab_new ();
 
-    manager->priv->bg = xfce_bg_new ();
-
     manager->priv->prefs = gs_prefs_new();
 
-    xfce_bg_load_from_preferences (manager->priv->bg, NULL);
-
     add_deepsleep_idle(manager);
 }
 
@@ -763,12 +702,8 @@ manager_show_window (GSManager *manager,
 
     manager_add_job_for_window (manager, window, job);
 
-    manager->priv->activate_time = time (NULL);
-
-    if (manager->priv->prefs->lock_timeout >= 0) {
-        remove_lock_timer (manager);
-        add_lock_timer (manager, manager->priv->prefs->lock_timeout);
-    }
+    remove_lock_timer (manager);
+    add_lock_timer (manager, manager->priv->prefs->lock_timeout);
 
     if (manager->priv->prefs->cycle >= 10000) {
         remove_cycle_timer (manager);
@@ -1040,10 +975,6 @@ gs_manager_finalize (GObject *object) {
 
     g_return_if_fail (manager->priv != NULL);
 
-    if (manager->priv->bg != NULL) {
-        g_object_unref (manager->priv->bg);
-    }
-
     remove_deepsleep_idle (manager);
     remove_timers(manager);
 
@@ -1054,7 +985,6 @@ gs_manager_finalize (GObject *object) {
     gs_manager_destroy_windows (manager);
 
     manager->priv->active = FALSE;
-    manager->priv->activate_time = 0;
 
     g_object_unref (manager->priv->grab);
 
@@ -1192,34 +1122,20 @@ gs_manager_deactivate (GSManager *manager) {
 
     /* reset state */
     manager->priv->active = FALSE;
-    manager->priv->activate_time = 0;
     manager->priv->dialog_up = FALSE;
 
-    gs_manager_set_lock_active (manager, FALSE);
+    gs_manager_enable_locker (manager, FALSE);
 
     return TRUE;
 }
 
 gboolean
-gs_manager_set_active (GSManager *manager,
-                       gboolean   active) {
-    gboolean res;
-
+gs_manager_activate_saver (GSManager *manager,
+                           gboolean   active) {
     if (active) {
-        res = gs_manager_activate (manager);
-    } else {
-        res = gs_manager_deactivate (manager);
+        return gs_manager_activate (manager);
     }
-
-    return res;
-}
-
-gboolean
-gs_manager_get_active (GSManager *manager) {
-    g_return_val_if_fail (manager != NULL, FALSE);
-    g_return_val_if_fail (GS_IS_MANAGER (manager), FALSE);
-
-    return manager->priv->active;
+    return gs_manager_deactivate (manager);
 }
 
 gboolean
