@@ -127,7 +127,7 @@ _gs_prefs_set_lock_timeout (GSPrefs *prefs,
     if (value > 480)
         value = 480;
 
-    prefs->lock_timeout = value * 60000;
+    prefs->lock_timeout = value;
 }
 
 static void
@@ -239,6 +239,12 @@ _gs_prefs_set_keyboard_command (GSPrefs    *prefs,
     }
 }
 
+static void
+_gs_prefs_set_keyboard_displayed (GSPrefs  *prefs,
+                                        gboolean  displayed) {
+    prefs->keyboard_displayed = displayed;
+}
+
 static void
 _gs_prefs_set_status_message_enabled (GSPrefs  *prefs,
                                       gboolean  enabled) {
@@ -354,6 +360,11 @@ gs_prefs_load_from_settings (GSPrefs *prefs) {
     _gs_prefs_set_keyboard_command (prefs, string);
     g_free (string);
 
+    bvalue = xfconf_channel_get_bool (prefs->priv->channel,
+                                      KEY_KEYBOARD_DISPLAYED,
+                                      DEFAULT_KEY_KEYBOARD_DISPLAYED);
+    _gs_prefs_set_keyboard_displayed (prefs, bvalue);
+
     bvalue = xfconf_channel_get_bool (prefs->priv->channel,
                                       KEY_STATUS_MESSAGE_ENABLED,
                                       DEFAULT_KEY_STATUS_MESSAGE_ENABLED);
@@ -385,6 +396,23 @@ gs_prefs_load_from_settings (GSPrefs *prefs) {
     _gs_prefs_set_user_switch_enabled (prefs, bvalue);
 }
 
+gchar *
+gs_prefs_get_theme_arguments (GSPrefs     *prefs,
+                              const gchar *theme) {
+    gchar *property;
+    gchar *arguments;
+    gchar *theme_name;
+
+    theme_name = g_utf8_substring (theme, 13, g_utf8_strlen(theme, -1));
+    property = g_strdup_printf ("/screensavers/%s/arguments", theme_name);
+    arguments = xfconf_channel_get_string (prefs->priv->channel, property, "");
+
+    g_free(theme_name);
+    g_free(property);
+
+    return arguments;
+}
+
 static void
 key_changed_cb (XfconfChannel *channel,
                 gchar         *property,
@@ -452,6 +480,11 @@ key_changed_cb (XfconfChannel *channel,
         command = xfconf_channel_get_string (channel, property, DEFAULT_KEY_KEYBOARD_COMMAND);
         _gs_prefs_set_keyboard_command (prefs, command);
         g_free (command);
+    } else if (strcmp (property, KEY_KEYBOARD_DISPLAYED) == 0) {
+        gboolean enabled;
+
+        enabled = xfconf_channel_get_bool (channel, property, DEFAULT_KEY_KEYBOARD_DISPLAYED);
+        _gs_prefs_set_keyboard_displayed (prefs, enabled);
     } else if (strcmp (property, KEY_STATUS_MESSAGE_ENABLED) == 0) {
         gboolean enabled;
 
