@@ -47,6 +47,7 @@
 #include "xfce4-screensaver-preferences-ui.h"
 
 #define GPM_COMMAND "xfce4-power-manager-settings"
+#define CONFIGURE_COMMAND "xfce4-screensaver-configure"
 
 enum {
     NAME_COLUMN = 0,
@@ -71,6 +72,7 @@ static gboolean        lock_delay_writable;
 static gboolean        keyboard_command_writable;
 static gboolean        logout_command_writable;
 static gboolean        logout_delay_writable;
+static gchar          *active_theme = NULL;
 
 static gint opt_socket_id = 0;
 static GOptionEntry entries[] = {
@@ -271,8 +273,11 @@ config_set_theme (const char *theme_id) {
         /* set the themes key to contain all available screensavers */
         strv = get_all_theme_ids (theme_manager);
     } else {
+        GtkWidget *configure_button = GTK_WIDGET (gtk_builder_get_object (builder, "configure_button"));
         mode = GS_MODE_SINGLE;
         strv = g_strsplit (theme_id, "%%%", 1);
+        active_theme = g_strdup (theme_id);
+        gtk_widget_set_sensitive (configure_button, TRUE);
     }
 
     config_set_mode (mode);
@@ -522,21 +527,44 @@ config_set_logout_command (const gchar *command) {
     xfconf_channel_set_string (screensaver_channel, KEY_LOGOUT_COMMAND, command);
 }
 
+static gchar *
+config_get_theme_arguments (const gchar *theme) {
+    gchar *property;
+    gchar *arguments;
+    gchar *theme_name;
+
+    theme_name = g_utf8_substring (theme, 13, g_utf8_strlen(theme, -1));
+    property = g_strdup_printf ("/screensavers/%s/arguments", theme_name);
+    arguments = xfconf_channel_get_string (screensaver_channel, property, "");
+
+    g_free(theme_name);
+    g_free(property);
+
+    return arguments;
+}
+
 static void
 job_set_theme (GSJob      *job,
                const char *theme) {
     GSThemeInfo *info;
-    const char  *command;
+    gchar       *command = NULL;
+    gchar       *arguments = NULL;
 
     command = NULL;
 
     info = gs_theme_manager_lookup_theme_info (theme_manager, theme);
     if (info != NULL) {
-        command = gs_theme_info_get_exec (info);
+        arguments = config_get_theme_arguments (theme);
+        command = g_strdup_printf ("%s %s", gs_theme_info_get_exec (info), arguments);
     }
 
     gs_job_set_command (job, command);
 
+    if (arguments)
+        g_free (arguments);
+    if (command)
+        g_free (command);
+
     if (info != NULL) {
         gs_theme_info_unref (info);
     }
@@ -734,6 +762,7 @@ tree_selection_next (GtkTreeSelection *selection) {
 static void
 tree_selection_changed_cb (GtkTreeSelection *selection,
                            GtkWidget        *preview) {
+    GtkWidget    *configure_button;
     GtkTreeIter   iter;
     GtkTreeModel *model;
     char         *theme;
@@ -743,6 +772,14 @@ tree_selection_changed_cb (GtkTreeSelection *selection,
         return;
     }
 
+    if (active_theme != NULL) {
+        g_free (active_theme);
+        active_theme = NULL;
+
+        configure_button = GTK_WIDGET (gtk_builder_get_object (builder, "configure_button"));
+        gtk_widget_set_sensitive (configure_button, FALSE);
+    }
+
     gtk_tree_model_get (model, &iter, ID_COLUMN, &theme, NAME_COLUMN, &name, -1);
 
     if (theme == NULL) {
@@ -757,6 +794,26 @@ tree_selection_changed_cb (GtkTreeSelection *selection,
     g_free (name);
 }
 
+static void
+configure_button_clicked_cb (GtkToolButton *button,
+                             gpointer       user_data) {
+    gchar    *configure_cmd;
+    GError   *error = NULL;
+    gboolean  res;
+
+    configure_cmd = g_strdup_printf ("%s %s", CONFIGURE_COMMAND, active_theme);
+
+    res = xfce_gdk_spawn_command_line_on_screen (gdk_screen_get_default (),
+                                                 configure_cmd,
+                                                 &error);
+    if (!res) {
+        g_warning ("Unable to start configure command: %s", error->message);
+        g_error_free (error);
+    }
+
+    g_free(configure_cmd);
+}
+
 static void
 idle_delay_value_changed_cb (GtkSpinButton *spin,
                                  gpointer  user_data) {
@@ -938,6 +995,44 @@ setup_treeview (GtkWidget *tree,
                       preview);
 }
 
+static void
+reload_theme (GtkWidget *treeview) {
+    GtkWidget        *preview;
+    GtkTreeIter       iter;
+    GtkTreeModel     *model;
+    GtkTreeSelection *selection;
+    char             *theme;
+    char             *name;
+
+    if (active_theme == NULL) {
+        return;
+    }
+
+    model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
+    if (model == NULL) {
+        return;
+    }
+
+    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
+
+    if (!gtk_tree_selection_get_selected (selection, &model, &iter)) {
+        return;
+    }
+
+    gtk_tree_model_get (model, &iter, ID_COLUMN, &theme, NAME_COLUMN, &name, -1);
+
+    if (theme == NULL) {
+        g_free (name);
+        return;
+    }
+
+    preview  = GTK_WIDGET (gtk_builder_get_object (builder, "saver_themes_preview_area"));
+    preview_set_theme (preview, theme, name);
+
+    g_free (theme);
+    g_free (name);
+}
+
 static void
 setup_treeview_selection (GtkWidget *tree) {
     char         *theme;
@@ -1386,6 +1481,10 @@ key_changed_cb (XfconfChannel *channel, const gchar *key, gpointer data) {
         GtkWidget *treeview;
         treeview = GTK_WIDGET (gtk_builder_get_object (builder, "saver_themes_treeview"));
         setup_treeview_selection (treeview);
+    } else if (g_str_has_suffix (key, "arguments")) {
+        GtkWidget *treeview;
+        treeview = GTK_WIDGET (gtk_builder_get_object (builder, "saver_themes_treeview"));
+        reload_theme (treeview);
     }
 }
 
@@ -1735,6 +1834,8 @@ configure_capplet (void) {
     GtkWidget *root_warning_infobar;
     GtkWidget *preview_button;
     GtkWidget *gpm_button;
+    GtkWidget *configure_toolbar;
+    GtkWidget *configure_button;
     GtkWidget *fullscreen_preview_window;
     GtkWidget *fullscreen_preview_area;
     GtkWidget *fullscreen_preview_previous;
@@ -1779,6 +1880,8 @@ configure_capplet (void) {
     root_warning_infobar        = GTK_WIDGET (gtk_builder_get_object (builder, "root_warning_infobar"));
     preview_button              = GTK_WIDGET (gtk_builder_get_object (builder, "preview_button"));
     gpm_button                  = GTK_WIDGET (gtk_builder_get_object (builder, "power_management_button"));
+    configure_button            = GTK_WIDGET (gtk_builder_get_object (builder, "configure_button"));
+    configure_toolbar           = GTK_WIDGET (gtk_builder_get_object (builder, "configure_toolbar"));
     fullscreen_preview_window   = GTK_WIDGET (gtk_builder_get_object (builder, "fullscreen_preview_window"));
     fullscreen_preview_area     = GTK_WIDGET (gtk_builder_get_object (builder, "fullscreen_preview_area"));
     fullscreen_preview_close    = GTK_WIDGET (gtk_builder_get_object (builder, "fullscreen_preview_close"));
@@ -1805,6 +1908,16 @@ configure_capplet (void) {
                       G_CALLBACK (key_changed_cb),
                       NULL);
 
+    if (!is_program_in_path (CONFIGURE_COMMAND)) {
+        gtk_widget_set_no_show_all (configure_toolbar, TRUE);
+        gtk_widget_hide (configure_toolbar);
+    } else {
+        g_signal_connect (configure_button,
+                          "clicked",
+                          G_CALLBACK (configure_button_clicked_cb),
+                          screensaver_channel);
+    }
+
     /* Idle delay */
     widget = GTK_WIDGET (gtk_builder_get_object (builder, "saver_idle_activation_delay"));
     delay = config_get_idle_delay (&idle_delay_writable);
@@ -1970,7 +2083,11 @@ configure_capplet (void) {
 
 static void
 finalize_capplet (void) {
-    g_object_unref (screensaver_channel);
+    if (screensaver_channel)
+        g_object_unref (screensaver_channel);
+
+    if (active_theme)
+        g_free (active_theme);
 }
 
 int
@@ -2060,8 +2177,11 @@ main (int    argc,
 
     finalize_capplet ();
 
-    g_object_unref (theme_manager);
-    g_object_unref (job);
+    if (theme_manager)
+        g_object_unref (theme_manager);
+
+    if (job)
+        g_object_unref (job);
 
     return 0;
 }
